"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var utils_1 = require("./utils");
function isMetadataFieldName(name) {
    return ['__schema', '__type'].includes(name);
}
function isRootType(type, schema) {
    return (graphql_1.isEqualType(type, schema.getQueryType()) ||
        graphql_1.isEqualType(type, schema.getMutationType()) ||
        graphql_1.isEqualType(type, schema.getSubscriptionType()));
}
var metadataFieldMap = {
    __schema: graphql_1.SchemaMetaFieldDef,
    __type: graphql_1.TypeMetaFieldDef
};
var SelectionSetToObject = /** @class */ (function () {
    function SelectionSetToObject(_scalars, _schema, _convertName, _addTypename, _parentSchemaType, _selectionSet) {
        this._scalars = _scalars;
        this._schema = _schema;
        this._convertName = _convertName;
        this._addTypename = _addTypename;
        this._parentSchemaType = _parentSchemaType;
        this._selectionSet = _selectionSet;
        this._primitiveFields = [];
        this._primitiveAliasedFields = [];
        this._linksFields = [];
        this._fragmentSpreads = [];
        this._inlineFragments = {};
        this._queriedForTypename = false;
    }
    SelectionSetToObject.prototype.createNext = function (parentSchemaType, selectionSet) {
        throw new Error("You must override createNext in your SelectionSetToObject implementation!");
    };
    SelectionSetToObject.prototype.wrapTypeWithModifiers = function (baseType, type) {
        throw new Error("You must override wrapTypeWithModifiers in your SelectionSetToObject implementation!");
    };
    SelectionSetToObject.prototype._collectField = function (field) {
        if (field.name.value === '__typename') {
            this._queriedForTypename = true;
            return;
        }
        if (graphql_1.isObjectType(this._parentSchemaType) || graphql_1.isInterfaceType(this._parentSchemaType)) {
            var schemaField = void 0;
            if (isRootType(this._parentSchemaType, this._schema) && isMetadataFieldName(field.name.value)) {
                schemaField = metadataFieldMap[field.name.value];
            }
            else {
                schemaField = this._parentSchemaType.getFields()[field.name.value];
            }
            var rawType = schemaField.type;
            var baseType = utils_1.getBaseType(rawType);
            var typeName = baseType.name;
            if (this._scalars[typeName] || graphql_1.isEnumType(baseType)) {
                if (field.alias && field.alias.value) {
                    this._primitiveAliasedFields.push({
                        fieldName: field.name.value,
                        alias: field.alias.value
                    });
                }
                else {
                    this._primitiveFields.push(field.name.value);
                }
            }
            else {
                var selectionSetToObject = this.createNext(baseType, field.selectionSet);
                this._linksFields.push({
                    alias: field.alias ? field.alias.value : null,
                    name: field.name.value,
                    type: typeName,
                    selectionSet: this.wrapTypeWithModifiers(selectionSetToObject.string, rawType)
                });
            }
        }
    };
    SelectionSetToObject.prototype._collectFragmentSpread = function (node) {
        this._fragmentSpreads.push(node.name.value);
    };
    SelectionSetToObject.prototype._collectInlineFragment = function (node) {
        var onType = node.typeCondition.name.value;
        var schemaType = this._schema.getType(onType);
        var selectionSet = this.createNext(schemaType, node.selectionSet);
        if (!this._inlineFragments[onType]) {
            this._inlineFragments[onType] = [];
        }
        this._inlineFragments[onType].push(selectionSet.string);
    };
    Object.defineProperty(SelectionSetToObject.prototype, "string", {
        get: function () {
            if (!this._selectionSet || !this._selectionSet.selections || this._selectionSet.selections.length === 0) {
                return '';
            }
            var selections = this._selectionSet.selections;
            for (var _i = 0, selections_1 = selections; _i < selections_1.length; _i++) {
                var selection = selections_1[_i];
                switch (selection.kind) {
                    case graphql_1.Kind.FIELD:
                        this._collectField(selection);
                        break;
                    case graphql_1.Kind.FRAGMENT_SPREAD:
                        this._collectFragmentSpread(selection);
                        break;
                    case graphql_1.Kind.INLINE_FRAGMENT:
                        this._collectInlineFragment(selection);
                        break;
                }
            }
            var parentName = this._convertName(this._parentSchemaType.name, {
                useTypesPrefix: true
            });
            var typeName = this._addTypename || this._queriedForTypename ? this.buildTypeNameField() : null;
            var baseFields = this.buildPrimitiveFields(parentName, this._primitiveFields);
            var aliasBaseFields = this.buildAliasedPrimitiveFields(parentName, this._primitiveAliasedFields);
            var linksFields = this.buildLinkFields(this._linksFields);
            var inlineFragments = this.buildInlineFragments(this._inlineFragments);
            var fragmentSpreads = this.buildFragmentSpread(this._fragmentSpreads);
            var fieldsSet = [typeName, baseFields, aliasBaseFields, linksFields, fragmentSpreads, inlineFragments].filter(function (f) { return f && f !== ''; });
            return this.mergeAllFields(fieldsSet);
        },
        enumerable: true,
        configurable: true
    });
    SelectionSetToObject.prototype.mergeAllFields = function (fieldsSet) {
        return utils_1.quoteIfNeeded(fieldsSet, ' & ');
    };
    SelectionSetToObject.prototype.buildTypeNameField = function () {
        var possibleTypes = [];
        if (!graphql_1.isUnionType(this._parentSchemaType) && !graphql_1.isInterfaceType(this._parentSchemaType)) {
            possibleTypes.push(this._parentSchemaType.name);
        }
        if (possibleTypes.length === 0) {
            return null;
        }
        return "{ " + this.formatNamedField('__typename') + (this._queriedForTypename ? '' : '?') + ": " + possibleTypes
            .map(function (t) { return "'" + t + "'"; })
            .join(' | ') + " }";
    };
    SelectionSetToObject.prototype.buildPrimitiveFields = function (parentName, fields) {
        if (fields.length === 0) {
            return null;
        }
        return "Pick<" + parentName + ", " + fields.map(function (field) { return "'" + field + "'"; }).join(' | ') + ">";
    };
    SelectionSetToObject.prototype.buildAliasedPrimitiveFields = function (parentName, fields) {
        var _this = this;
        if (fields.length === 0) {
            return null;
        }
        return "{ " + fields
            .map(function (aliasedField) { return _this.formatNamedField(aliasedField.alias) + ": " + parentName + "['" + aliasedField.fieldName + "']"; })
            .join(', ') + " }";
    };
    SelectionSetToObject.prototype.formatNamedField = function (name) {
        return name;
    };
    SelectionSetToObject.prototype.buildLinkFields = function (fields) {
        var _this = this;
        if (fields.length === 0) {
            return null;
        }
        return "{ " + fields
            .map(function (field) { return _this.formatNamedField(field.alias || field.name) + ": " + field.selectionSet; })
            .join(', ') + " }";
    };
    SelectionSetToObject.prototype.buildInlineFragments = function (inlineFragments) {
        var allPossibleTypes = Object.keys(inlineFragments).map(function (typeName) { return inlineFragments[typeName].join(' & '); });
        return allPossibleTypes.length === 0 ? null : "(" + allPossibleTypes.join(' | ') + ")";
    };
    SelectionSetToObject.prototype.buildFragmentSpread = function (fragmentsSpread) {
        var _this = this;
        if (fragmentsSpread.length === 0) {
            return null;
        }
        return utils_1.quoteIfNeeded(fragmentsSpread.map(function (fragmentName) {
            return _this._convertName(fragmentName, {
                suffix: 'Fragment',
                useTypesPrefix: true
            });
        }), ' & ');
    };
    return SelectionSetToObject;
}());
exports.SelectionSetToObject = SelectionSetToObject;
//# sourceMappingURL=selection-set-to-object.js.map